const Plays = require("./Agent/Plays");
const Modules = require("./Agent/Modules");

const ISCSIDConfiguration = require("./Agent/Models/ISCSIDConfiguration");
const ISCSIInitiatorName = require("./Agent/Models/ISCSIInitiatorName");
const ISCSIDriveCredentials = require("./Agent//Models/ISCSIDriveCredentials");
const FilesystemInformation = require("./Agent/Models/FilesystemInformation");
const ISCSINameValidator = require("validators").ISCSIName.ISCSINameValidator;

(async() =>
{	
	console.log(ISCSINameValidator.assertValid("iqn.2018-05.com.bigstep.storage.instance-539"));
	await instance_agent_iscsi_setup_and_login();

	//Test values
	const objDriveCredentials = {
		"drive_id": 4,
		"lun_id": 1,
		"storage_ip_address": "192.168.137.2",
		"storage_port": 3260,
		"target_iqn": "iqn.2013-01.com.bigstep:storage.zkt1oxb.ajeb1n3.60eqwne"
	};

	const objPartitionTable = {
		"type": "gpt",
		"partitions": [
			{
				"index": 1
			}
		]
	};

	const objFilesystemInformation = {
		"filesystem_type": "ext4",
		"block_size": 1024
	};
	
	const strMountPoint = "/mnt/drive-4";

	await instance_agent_drive_partition(objDriveCredentials, objPartitionTable);
	await instance_agent_drive_partition_format(objDriveCredentials, /*nPartitionIndex*/ 1, objFilesystemInformation);
	await instance_agent_drive_partition_mount(objDriveCredentials, /*nPartitionIndex*/ 1, objFilesystemInformation, strMountPoint);
	await instance_agent_disk_partition_unmount(strMountPoint);
})();

async function instance_agent_iscsi_setup_and_login()
{
	/**
	 * objNodeInformation contains the node information
	 * @TODO: Get the real info
	*/
	let objNodeInformation = [];
	
	objNodeInformation["iscsi_initiator_iqn"] = "iqn.2018-06.com.bigstep.storage.instance-1";
	objNodeInformation["isns_address"] = "192.168.137.2";
	objNodeInformation["isns_port"] = 3205;
	objNodeInformation["iscsi_authmethod"] = "CHAP";
	objNodeInformation["iscsi_username"] = "4f788wPmrcHtY";
	objNodeInformation["iscsi_password"] = "4m5CecwWYR3Lg"; //this should be the result of the decryption
	/**
	 * Encrypted password is: "Use bsidev.password_decrypt:eyJycSI6ImJyLmYtXzByOHRPWGlLbUZ5cTk3ZG9XZzlNMG8wQTdScnRjZTVoVi1JNThUWGNkRUVkN3ZTYUJ
	 * kY0o0YU9acWtVQVBVWEd2cEYtaEZhSUkxN1VLWENBSXQyOWV3YVV1TnFWbWN2eWpTcFUwTUExeE5TT2E2eERFby1IRGZaRUtTaGhNIiwidiI6Ijc2VUt5Q1lweC1LdUc1SjlJTVJDR1EifQ"
	 * 
	 */
	//These are hardcoded for the drive auto-login play
	objNodeInformation["nr_sessions"] = 1;
	objNodeInformation["startup"] = "automatic";


	const nodeInformation = new ISCSIDConfiguration(
		objNodeInformation["isns_address"],
		objNodeInformation["isns_port"],
		objNodeInformation["iscsi_authmethod"],
		objNodeInformation["iscsi_username"],
		objNodeInformation["iscsi_password"],
		objNodeInformation["nr_sessions"],
		objNodeInformation["startup"]
	);

	const ISCSIInitiatorInformation = new ISCSIInitiatorName(
		objNodeInformation["iscsi_initiator_iqn"]
	);

	const objParams = {
		"node_information": nodeInformation,
		"iscsi_initiator_name": ISCSIInitiatorInformation
	};

	const bIsChanged = await Plays.ISCSISetupAndLoginPlaySingleton.run(objParams);

	let strResponse = null;

	if(bIsChanged)
	{
		strResponse = "ISCSI setup and login completed successfully.";
	}
	else
	{
		strResponse = "ISCSI setup and login already completed.";
	}

	return strResponse;
}

async function instance_agent_drive_partition(objDriveCredentials, objPartitionTable)
{
	//objDriveCredentials must have: drive_id, lun_id, storage_ip_address, storage_port and target_iqn
	const diskCredentials = new ISCSIDriveCredentials(
		objDriveCredentials["drive_id"],
		objDriveCredentials["lun_id"],
		objDriveCredentials["storage_ip_address"],
		objDriveCredentials["storage_port"],
		objDriveCredentials["target_iqn"]
	);

	const driveModule = Modules.DriveSingleton;

	const moduleResponse = driveModule.getDiskPath(diskCredentials, false);
	const strDiskPath = moduleResponse.getOutput();

	return await instance_agent_disk_partition(strDiskPath, objPartitionTable);
}

async function instance_agent_disk_partition(strDiskPath, objPartitionTable)
{
	const diskModule = Modules.DiskSingleton;

	//objPartitionTable must have: type and partitions
	const partitionTable = diskModule.toPartitionTable(objPartitionTable);

	const objParams = {
		"disk_path": strDiskPath,
		"partition_table": partitionTable
	};

	await Plays.DiskPartitionPlaySingleton.run(objParams);

	return strDiskPath + " partitioned successfully!";
}

async function instance_agent_drive_partition_format(objDriveCredentials, nPartitionIndex, objFilesystemInformation)
{
	//objDriveCredentials must have: drive_id, lun_id, storage_ip_address, storage_port and target_iqn
	const diskCredentials = new ISCSIDriveCredentials(
		objDriveCredentials["drive_id"],
		objDriveCredentials["lun_id"],
		objDriveCredentials["storage_ip_address"],
		objDriveCredentials["storage_port"],
		objDriveCredentials["target_iqn"]
	);

	const moduleResponse = Modules.DriveSingleton.getDiskPath(diskCredentials, false);
	const strPartitionPath = moduleResponse.getOutput() + "-part" + nPartitionIndex;

	return await instance_agent_disk_partition_format(strPartitionPath, objFilesystemInformation);
}

async function instance_agent_disk_partition_format(strDiskPartitionPath, objFilesystemInformation)
{
	const filesystemInformation = new FilesystemInformation(
		objFilesystemInformation["filesystem_type"],
		objFilesystemInformation["block_size"]
	);

	const objParams = {
		"disk_partition_path": strDiskPartitionPath,
		"filesystem_information": filesystemInformation
	};

	return await Plays.DiskPartitionFormatPlaySingleton.run(objParams);
}

async function instance_agent_drive_partition_mount(objDriveCredentials, nPartitionIndex, objFilesystemInformation, strMountPoint = null)
{
	//objDriveCredentials must have: drive_id, lun_id, storage_ip_address, storage_port, target_iqn
	const driveCredentials = new ISCSIDriveCredentials(
		objDriveCredentials["drive_id"],
		objDriveCredentials["lun_id"],
		objDriveCredentials["storage_ip_address"],
		objDriveCredentials["storage_port"],
		objDriveCredentials["target_iqn"]
	);

	const driveModule = Modules.DriveSingleton;

	/**
	 * Partition mount point is automatically generated if not sent as the last parameter.
	 * Generated mount point has the format "/mnt/drive{drive_id}".
	 */
	const strPartitionMountPoint = strMountPoint === null ? driveModule.getMountPointPath(driveCredentials.driveID) : strMountPoint;

	//Get the unresolved symlink from /dev/disk/by-path
	const moduleResponse = driveModule.getDiskPath(driveCredentials, false);
	const strDrivePartitionPath = moduleResponse.getOutput() + "-part" + nPartitionIndex;

	return await instance_agent_disk_partition_mount(strDrivePartitionPath, objFilesystemInformation, strPartitionMountPoint);
}

async function instance_agent_disk_partition_mount(strDiskPartitionPath, objFilesystemInformation, strMountPoint)
{
	//objFilesystemInformation needs to be an object sent by BSI, that holds filesystem_type and block_size
	const filesystemInformation = new FilesystemInformation(
		objFilesystemInformation["filesystem_type"],
		objFilesystemInformation["block_size"]
	);

	const objParams = {
		"disk_partition_path": strDiskPartitionPath,
		"filesystem_information": filesystemInformation,
		"mount_point": strMountPoint
	};

	await Plays.DiskPartitionAutomaticMountPlaySingleton.run(objParams);

	return "Successfully mounted partition " + strDiskPartitionPath + " at " + strMountPoint;
}

async function instance_agent_disk_partition_unmount(strMountPoint)
{
	const objParams = {
		"mount_point": strMountPoint
	};

	await Plays.DiskPartitionAutomaticUnmountPlaySingleton.run(objParams);

	return "Successfully unmounted partition at mount point " + strMountPoint;
}
